<?php namespace Lunovus\ExternalStore;

class Update
{
    const CACHE_ALLOWED = (LUNOVUS_EXTERNAL_STORE__ENVIRONMENT == 'production');

    public static function init()
    {
        add_filter('plugins_api', [get_called_class(), 'info'], 20, 3);
        add_filter('site_transient_update_plugins', [get_called_class(), 'update']);
        add_action('upgrader_process_complete', [get_called_class(), 'purge'], 10, 2);
    }

    /**
     * Returns latest plugin version info for update checker
     */
    public static function info($result, $action, $args)
    {
        // Not information request
        if ($action !== 'plugin_information')
        {
            return $result;
        }

        // Not this plugin slug
        if ($args->slug !== LUNOVUS_EXTERNAL_STORE_NAME)
        {
            return $result;
        }
        
        if ($remote = self::requestInfo())
        {
            return self::transformInfo($remote);
        }

        return $result;
    }

    /**
     * Updates the plugin with newest version
     */
    public static function update($transient)
    {
        if (empty($transient->checked))
        {
            return $transient;
        }

        $remote = self::requestInfo();

        if ($remote
            && version_compare(LUNOVUS_EXTERNAL_STORE_VERSION, $remote->version, '<')
            && version_compare($remote->requires, get_bloginfo('version'), '<=')
            && version_compare($remote->requires_php, PHP_VERSION, '<')
        ) {
            $result = new \stdClass();
            $result->slug = LUNOVUS_EXTERNAL_STORE_NAME;
            $result->plugin = LUNOVUS_EXTERNAL_STORE_NAME . '/' . LUNOVUS_EXTERNAL_STORE_NAME . '.php';
            $result->new_version = $remote->version;
            $result->tested = $remote->tested;
            $result->package = $remote->download_url;

            $transient->response[$result->plugin] = $result;
        }

        return $transient;
    }

    /**
     * Removes plugin transient data
     */
    public static function purge($upgrader, $options)
    {
        if (self::CACHE_ALLOWED
            && ($options['action'] === 'update')
            && ($options['type'] === 'plugin')
        ) {
            // Clean the cache when new plugin version is installed
            delete_transient(LUNOVUS_EXTERNAL_STORE_CACHE_KEY);
        }
    }

    /**
     * Requests info about lastest version
     */
    public static function requestInfo()
    {
        $remote = get_transient(LUNOVUS_EXTERNAL_STORE_CACHE_KEY);

        if ( ! $remote || ! self::CACHE_ALLOWED)
        {
            $remote = wp_remote_get(
                LUNOVUS_EXTERNAL_STORE__UPDATE_URL, [
                    'timeout' => 10,
                    'headers' => [
                        'Accept' => 'application/json'
                    ]
                ]
            );

            if (is_wp_error($remote)
                || (wp_remote_retrieve_response_code($remote) !== 200)
                || empty(wp_remote_retrieve_body($remote))
            ) {
                return false;
            }

            set_transient(
                LUNOVUS_EXTERNAL_STORE_CACHE_KEY, $remote, DAY_IN_SECONDS
            );
        }

        $remote = json_decode(wp_remote_retrieve_body($remote));

        return $remote;
    }

    /**
     * Transforms lastest version info for update checker
     */
    public static function transformInfo($data)
    {
        $result = new \stdClass();

        $result->name = $data->name;
        $result->slug = $data->slug;
        $result->version = $data->version;
        $result->tested = $data->tested;
        $result->requires = $data->requires;
        $result->author = $data->author;
        $result->author_profile = $data->author_profile;
        $result->download_link = $data->download_url;
        $result->trunk = $data->download_url;
        $result->requires_php = $data->requires_php;
        $result->last_updated = $data->last_updated;

        $result->sections = [
            'description' => $data->sections->description,
            'installation' => $data->sections->installation,
            'changelog' => $data->sections->changelog
        ];

        return $result;
    }
}
